#========================================================================================
# package      DE_EPAGES::Ebay::API::WebService::EbayConfigService
# state        public
#----------------------------------------------------------------------------------------
# description  web service module for configuring ebay cartridge on a store db
#========================================================================================
package DE_EPAGES::Ebay::API::WebService::EbayConfigService;
use base DE_EPAGES::WebService::API::WebService::BaseService;

use strict;
use Config;
use DateTime::Duration;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( GetLog LogDebug );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Ebay::API::Object::RemoteRESTEbay;
use DE_EPAGES::Ebay::API::Table::EbayCallType qw ( GetAllInfoEbayCallTypes );
use DE_EPAGES::Object::API::Factory qw (
    JoinObjectPath
    LoadObject
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::XML::API::DateTimeFormatXML;
use Errno qw ( EAGAIN );

#ERRORDEFINITION
sub ErrorDefinition { return {
    'ForkError' => 'Weird Fork Error: #Msg',
}}
#ERRORDEFINITION END

#========================================================================================
# function     getEbayEnvironment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ebayconfigservice->getEbayEnvironment();
#----------------------------------------------------------------------------------------
# description  Gets info for all ebay environments on the store
#----------------------------------------------------------------------------------------
# return       $aValues | tleHash of each EbayEnvironment | ref.array.hash
#========================================================================================
sub getEbayEnvironment {
    my $self = shift;

    my $System = LoadRootObject();

    my $environments = $System->get('EbayEnvironments');

    my $aValues = [];

    foreach my $environment (@$environments) {
        my $hValue = {};

        $hValue = $environment->get([qw(ID Name Alias EbaySiteId IsSandBox
                                        CurrencyID LanguageID IsVisible IconPath
                                        ApiUrl OAuthGrandApplicationAccessEndpoint OAuthTokenRequestEndpoint
                                        CategoryListVersion CategoryListCount CategoryListIsUpdating
                                        CategoryListUpdateTime CategoryListUpdateTimeEbay )]);

        my $parser = DE_EPAGES::XML::API::DateTimeFormatXML->new();
        $hValue->{'CategoryListUpdateTime'} = ($hValue->{'CategoryListUpdateTime'})?
                                              $parser->format_datetime($hValue->{'CategoryListUpdateTime'}):'';
        $hValue->{'CategoryListUpdateTimeEbay'} = ($hValue->{'CategoryListUpdateTimeEbay'})?
                                                  $parser->format_datetime($hValue->{'CategoryListUpdateTimeEbay'}):'';

        push @$aValues, $hValue;
    }

    return $aValues;
}

#========================================================================================
# function     checkEbayToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($self->checkEbayToken( 'US' )) {}
#----------------------------------------------------------------------------------------
# description  Checks Authentication of an EbayEnvironment
#----------------------------------------------------------------------------------------
# input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
# return       $result | successful? | boolean
#========================================================================================
sub checkEbayToken {
    my $self = shift;

    my ($EnvironmentAlias) = @_;
    my $Environment;
    $Environment= LoadObjectByPath(JoinObjectPath(['','EbayEnvironments',$EnvironmentAlias])) if $EnvironmentAlias;

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('Environment' => $Environment);

    my $hCheckResult = $RemoteEbay->getCategories('1');

    return ($hCheckResult)?1:0;
}

#========================================================================================
# function     getOAuthTokensByAuthCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getOAuthTokensByAuthCode( $AuthCode, 'US' );
#----------------------------------------------------------------------------------------
# description  fetch a token from eBay
#----------------------------------------------------------------------------------------
# input        $AuthCode    | authentication code for fetching tokens | string
# input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
# return       $result | result set (with token) | ref hash
#========================================================================================
sub getOAuthTokensByAuthCode {
    my $self = shift;
    my ($AuthCode , $EnvironmentAlias) = @_;
    my $Environment;
    $Environment= LoadObjectByPath(JoinObjectPath(['','EbayEnvironments',$EnvironmentAlias])) if $EnvironmentAlias;

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('Environment' => $Environment);
    my $result = $RemoteEbay->getOAuthTokensByAuthCode($AuthCode);

    return $result;
}

#========================================================================================
# function     getOAuthTokensByAuthCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getOAuthTokensByAuthCode( $AuthCode, 'US' );
#----------------------------------------------------------------------------------------
# description  fetch a token from eBay
#----------------------------------------------------------------------------------------
# input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
# return       $result | result set (with token) | ref hash
#========================================================================================
sub getApplicationOAuthToken {
    my $self = shift;
    my ($EnvironmentAlias) = @_;
    my $Environment;

    $Environment= LoadObjectByPath(JoinObjectPath(['','EbayEnvironments',$EnvironmentAlias])) if $EnvironmentAlias;

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('Environment' => $Environment);
    my $result = $RemoteEbay->getApplicationOAuthToken();

    return $result;
}

#========================================================================================
# function     saveOAuthToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->saveOAuthToken( $hTokenInfo, 'US' );
#----------------------------------------------------------------------------------------
# description  fetch a token from eBay
#----------------------------------------------------------------------------------------
# input        $hTokenInfo    | tokens with info | ref hash
# input        $EnvironmentAlias | Identifier of an EbayEnvironment | string
#========================================================================================
sub saveOAuthToken {
    my $self = shift;
    my ($hTokenInfo , $EnvironmentAlias) = @_;
    my $Environment;
    $Environment= LoadObjectByPath(JoinObjectPath(['','EbayEnvironments',$EnvironmentAlias])) if $EnvironmentAlias;

    my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('Environment' => $Environment);
    $RemoteEbay->saveOAuthToken($hTokenInfo);

    return;

}

#========================================================================================
# function     getCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getCategories( $env->id );
#----------------------------------------------------------------------------------------
# description  Starts asynchronous category download for given environment
#----------------------------------------------------------------------------------------
# input        $aEnvironmentIDs | list of ObjectIDs of EbayEnvironments | ref.array.int
# return       $result | request successful submitted? | boolean
#========================================================================================
sub getCategories {
    my $self = shift;
    my ($aEnvironmentIDs) = @_;

    LogDebug("EbayConfigWebService getCategories: EnvironmentIDs", $aEnvironmentIDs);

    my $Environments;

    if (ref $aEnvironmentIDs eq 'ARRAY') {
        $Environments = join '', map { ' -environment '. LoadObject($_)->alias } @$aEnvironmentIDs;
    } else {
        $Environments = ' -environment '. LoadObject($aEnvironmentIDs)->alias;
    }

    Error('NoEnvironmentFound') unless $Environments;

    my $System = LoadRootObject();

    if ($Config{osname} eq 'MSWin32') {
        #ONLY WINDOWS
        my $cmd = 'start '.$ENV{'EPAGES_PERL'}.'\bin\perl '.$ENV{'EPAGES_CARTRIDGES'}.
                  '/DE_EPAGES/Ebay/Scripts/updateCategories.pl '.
                  ' -storename '.$System->get('StoreName').
                  $Environments.
                  ' -force -nooutput';

        GetLog()->debug("call script updateCategories on windows: $cmd");
        system( $cmd );
    } else {
        # do FORK for all other systems
        FORK : {
            if (my $pid = fork) {
                #parent here
                return 1;
            } elsif (defined $pid) {
                #child here
                my $cmd = $ENV{PERL}.' '.$ENV{'EPAGES_CARTRIDGES'}.
                          '/DE_EPAGES/Ebay/Scripts/updateCategories.pl '.
                          ' -storename '.$System->get('StoreName').
                          $Environments.
                          ' -force -nooutput';

                GetLog()->debug("call script updateCategories on non-windows-system: $cmd");
                exec( $cmd );

            } elsif ($! == EAGAIN) {
                sleep(5);
                redo FORK;
            } else {
                Error('ForkError', {'Msg' => $!});
            }
        }
    }
    return 0;
}

#========================================================================================
# function     getCallTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getCallTypes();
#----------------------------------------------------------------------------------------
# description  Get all Call Types
#----------------------------------------------------------------------------------------
# return       $result | array of CallTypeInfos | ref.array.hash
#========================================================================================
sub getCallTypes {
    my $self = shift;

    return GetAllInfoEbayCallTypes();
}

#========================================================================================
# function     getCallStatistics
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getCallStatistics( $hArgs );
#----------------------------------------------------------------------------------------
# description  Get the call statistics for the input args
#----------------------------------------------------------------------------------------
# input        $hArgs | with following keys:
#               <ul>
#                   <li>DateFrom        - start date - datetime string
#                   <li>DateTo          - end date   - datetime string
#                   <li>Interval        - interval of statistics periods - integer
#                   <li>IntervalType    - type of interval (e.g. 'days', 'weeks') - string
#                   <li>EbayCallType    - id of call type - integer
#                   <li>EbayEnvironment - id of ebayenvironment - integer
#               </ul> | ref.hash
# return       $result | array of results:
#                       <ul>
#                           <li>StartDate - start date of statistics period - datetime string
#                           <li>Count - result for this period - integer
#                       </ul> | ref.array.hash
#========================================================================================
sub getCallStatistics {
    my $self = shift;
    my ($hFormValues) = @_;

    my $parser = DE_EPAGES::XML::API::DateTimeFormatXML->new();

    my $startDate = $parser->parse_datetime($hFormValues->{'DateFrom'});
    my $endDate = $parser->parse_datetime($hFormValues->{'DateTo'});
    my $interval = $hFormValues->{'Interval'};
    my $intervalType = $hFormValues->{'IntervalType'};
    my $ebaycalltype = $hFormValues->{'EbayCallType'};
    my $ebayenvironment = $hFormValues->{'EbayEnvironment'};

    my $dbi = GetCurrentDBHandle();
    my @result;

    my $currentStartDate = $startDate;
    do {
        my $currentEndDate = $currentStartDate + DateTime::Duration->new(lc $intervalType => $interval);

        my @VariableList = ($currentStartDate, $currentEndDate);
        my @TypeList = ('Integer', 'Integer');
        my $additionalWhere = '';

        if ($ebaycalltype) {
            $additionalWhere .= " and ebaycalltypeid = ? ";
            push @VariableList , $ebaycalltype;
            push @TypeList , 'Integer';
        }

        if ($ebayenvironment) {
            $additionalWhere .= ' and ebayenvironmentid = ? ';
            push @VariableList, $ebayenvironment;
            push @TypeList , 'Integer';
        }

        ##no critic(ProhibitVariablesInSQLStatements)
        my $result = $dbi->execute("SELECT count(*) FROM ebaycall WHERE date > ? and date < ? $additionalWhere",  \@VariableList, \@TypeList );
        ##use critic
        push @result, {
            'StartDate' => $parser->format_datetime($currentStartDate),
            'Count' => $result->[0]->[0],
        };
        $currentStartDate = $currentEndDate;
    } while ($currentStartDate < $endDate);

    return \@result;
}

1;
