#========================================================================================
# package      DE_EPAGES::MailType::API::Object::MailType
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for MailType (aka e-mail events).
# example      #send order confirmation mail to customer
#               my $MailType = $Order->getSite()->child('MailTypes')->child('CustomerOrderConfirmation');
#
#               # create and send mail
#               my %MailParams = (
#                   'HTMLAllowed' => $Order->get('Customer')->get('IsHtmlEMailAllowed'),
#               );
#               my $Email = $Order->get('BillingAddress')->get('EMail');
#               $MailParams{'TO'} = $Email if defined $Email;
#
#               my %Vars = ();
#               my $Container = $Order->get('LineItemContainer');
#               my %RegionalParams = (
#                   Locale      => $Container->get('LocaleID'),
#                   Language    => GetCodeByLanguageID($Container->get('LanguageID')),
#               );
#               $MailType->send( $Order, \%MailParams, \%RegionalParams, \%Vars );
#========================================================================================
package DE_EPAGES::MailType::API::Object::MailType;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use DE_EPAGES::Core::API::Html qw ( ReplaceURIWithURL );
use DE_EPAGES::Core::API::TestParameter qw (TestHash TestObject);
use DE_EPAGES::Core::API::String qw(SplitEMailAddresses);
use DE_EPAGES::Mail::API::Mail;
use DE_EPAGES::MailType::API::Object::SmtpConnection;
use DE_EPAGES::Object::API::Language qw (  GetPKeyLanguageByCode );
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw (ReplaceTLEs );
use DE_EPAGES::Presentation::API::Template qw ( ProcessPage );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $MailType->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'MailType' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $MailType->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $MailType = DE_EPAGES::MailType::API::Object::MailType->insert($hInfo);
# example      my $Class = LoadClassByAlias('MailType');
#               my $MailType = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertMailType' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $MailType | new object | object
# hook         OBJ_InsertMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>MailTypeID    - mail type id         - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertMailType', {'Object' => $self, 'MailTypeID' => $self->id });
    delete $hInfo->{$_} foreach qw ( MailTypeTemplate IsActive FROM TO CC BCC ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $MailType->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteMailType' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>MailTypeID    - mail type id         - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteMailType', {'Object' => $self, 'MailTypeID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $MailType->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateMailType' and 'OBJ_AfterUpdateMailType'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>MailTypeID    - mail type id         - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>MailTypeID    - mail type id         - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateMailType', {'Object'=>$self, 'Values'=> $hValues, 'MailTypeID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateMailType',  {'Object'=>$self, 'Values'=> $hValues, 'MailTypeID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     send
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Sent = $MailType->send($Object, $hMailParams, $hRegionalParams, $hVars );
# example      $MailType->send( $User, {
#                       HTMLAllowed => 1,
#                       TO => $User->get('EMail'),
#                   }, {
#                       Language => 'de',
#                       Locale => 'de_DE'
#                   }, { } );
#----------------------------------------------------------------------------------------
# description  Sends an e-mail object using the current mail type if the MailType is
#               active or not.
#               The TLE placeholders in the mail templates for Content, Subject and
#               Signature are replaced by the variables given in $hVars.
#               Triggers the hook "MailTypeSend" before actually building the mail
#               object and sendinf the mail. Hook functions can prevent the mail from
#               beeing sent by setting the hook parameter "SendMail" to 0. In this case
#               the return value will be 0.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $hMailParams | e-mail options, see <function getMail> | ref.hash
# input        $hRegionalParams | regional options, see <function getMail> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# return       $Sent | true if the mail was actually sent | boolean
#========================================================================================
sub send {
    my $self = shift;
    my ($Object, $hMailParams, $hRegionalParams, $hVars ) = @_;
    TestHash( 'hMailParams', $hMailParams );
    TestHash( 'hRegionalParams', $hRegionalParams );
    TestHash( 'hVars', $hVars );

    my %Params = (
        'MailType' => $self,
        'Object' => $Object,
        'MailParams' => $hMailParams,
        'RegionalParams' => $hRegionalParams,
        'Vars' => $hVars,
        'SendMail' => 1,
    );
    TriggerHook('MailTypeSend', \%Params);
    return 0 unless $Params{'SendMail'};

    my $Mail = $self->getMail( $Object, $hMailParams, $hRegionalParams, $hVars);

    $self->sendMail( $Mail );
    return 1;
}

#========================================================================================
# function     sendMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MailType->sendMail( $Mail );
#----------------------------------------------------------------------------------------
# description  Sends the e-mail using the default SMTP connection that is configured in
#               the current database.
#----------------------------------------------------------------------------------------
# input        $Mail | e-mail object, see <package DE_EPAGES::Mail::API::Mail> | object
#========================================================================================
sub sendMail {
    my $self = shift;
    my ( $Mail) = @_;
    TestObject( 'Mail', $Mail );

    my $Connection = DE_EPAGES::MailType::API::Object::SmtpConnection->new;
    $Connection->connection( sub {
        $Connection->send( $Mail->createMail(), $Mail->getHeader() );
    });

    return;
}

#========================================================================================
# function     _getMailParams
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hMailParams = $MailType->_getMailParams($Object, $hParams, $hRegionalParams, $hVars);
#----------------------------------------------------------------------------------------
# description  Appends default mail type parameters to current mail parameter.
#----------------------------------------------------------------------------------------
# input        $Object | context of the e-mail, for example a shop or an order | object
# input        $hParams | e-mail options
#                       <ul>
#                         <li>FROM      - FROM       (optional) - string</li>
#                         <li>REPLY     - REPLY      (optional) - string</li>
#                         <li>TO        - TO         (optional) - string</li>
#                         <li>CC        - CC         (optional) - string</li>
#                         <li>BCC       - BCC        (optional) - string</li>
#                         <li>HTMLAllowed - send mail as HTML (optional, default: 0=plain text) - boolean</li>
#                         <li>Subject   - Subject of mail (optional) - string</li>
#                         <li>Content   - content of mail (optional) - string</li>
#                       </ul> | ref.hash optional
# input        $hRegionalParams | regional options
#                       <ul>
#                         <li>Locale</li>
#                         <li>Language</li>
#                       </ul> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# return       $hMailParams | e-mail options
#                       <ul>
#                         <li>FROM      - FROM      - string</li>
#                         <li>REPLY     - REPLY     - string</li>
#                         <li>TO        - TO        - string</li>
#                         <li>CC        - CC        - string</li>
#                         <li>BCC       - BCC       - string</li>
#                         <li>HTMLAllowed - send mail as HTML - boolean</li>
#                         <li>Subject   - subject of mail - string</li>
#                         <li>Content   - content of mail - string</li>
#                         <li>SkipBadRecipients - 1/0 ignore recipient addresses like 'name@epa@ges.de'- boolean</li>
#                         <li>Attachments   - attachments of mail - ref.array.hash</li>
#                       </ul> | ref.hash
#========================================================================================
sub _getMailParams {
    my $self = shift;
    my ($Object, $hParams, $hRegionalParams, $hVars) = @_;
    $hParams = {} unless defined $hParams;

    my %MailParams;
    $MailParams{'FROM'}        = $self->from($hParams->{'FROM'});
    $MailParams{'REPLY'}       = $self->_reply($hParams->{'REPLY'});
    $MailParams{'TO'}          = $self->to($hParams->{'TO'});
    $MailParams{'CC'}          = $self->_cc($hParams->{'CC'});
    $MailParams{'BCC'}         = $self->_bcc($hParams->{'BCC'});
    $MailParams{'HTMLAllowed'} = $self->_htmlAllowed($hParams->{'HTMLAllowed'});

    $MailParams{'Subject'}     = $self->_subject($Object,$hParams->{'Subject'}, $hRegionalParams, $hVars);
    $MailParams{'Content'}     = $self->content($Object,$hParams->{'Content'}, $hRegionalParams, $hVars);

    $MailParams{'Attachments'} = $hParams->{'Attachments'};
    $MailParams{'SkipBadRecipients'} = $hParams->{'SkipBadRecipients'};

    return \%MailParams
}

#========================================================================================
# function     getMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail = $MailType->getMail($Object, $hParams, $hRegionalParams, $hVars);
#----------------------------------------------------------------------------------------
# description  Creates an e-mail object for the mail type.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $hMailParams | e-mail options
#                       <ul>
#                         <li>FROM      - FROM       (optional) - string</li>
#                         <li>REPLY     - REPLY      (optional) - string</li>
#                         <li>TO        - TO         (optional) - string</li>
#                         <li>CC        - CC         (optional) - string</li>
#                         <li>BCC       - BCC        (optional) - string</li>
#                         <li>HTMLAllowed - send mail as HTML (default: 0=plain text) - boolean</li>
#                         <li>Subject   - subject of mail (optional) - string</li>
#                         <li>Content   - content of mail (optional) - string</li>
#                         <li>Attachments - mail attachments (optional) - ref.array.hash</li>
#                         <li>SkipBadRecipients - 1/0 ignore recipient addresses like 'name@epa@ges.de' (default: 1)- boolean
#                       </ul> | ref.hash optional
# input        $hRegionalParams | regional options
#                       <ul>
#                         <li>Locale - locale id, e.g. 'en_GB' - strin</li>
#                         <li>Language - language id, e.g. 'en' - string</li>
#                       </ul> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# return       $Mail | mail object (instance of <package DE_EPAGES::Mail::API::Mail>) | object
# hook         MailTypeGetMail | hook parameter keys :
#               <ul>
#                 <li>Object      - mail processed for object - object
#                 <li>MailType    - mail type - object
#                 <li>Mail        - mail - object
#               </ul>
#========================================================================================
sub getMail {
    my $self = shift;
    my ($Object, $hParams, $hRegionalParams, $hVars) = @_;

    TestHash( 'hParams', $hParams );

    $hParams = $self->_getMailParams($Object, $hParams, $hRegionalParams, $hVars);
    $hParams->{'SkipBadRecipients'} = 1 unless defined $hParams->{'SkipBadRecipients'};

    # create the mail, use utf-8 as mail encoding
    my $hInfo = {'HEADER_ENC'        => 'utf-8',
                 'SUBJECT'           => defined $hParams->{'Subject'} ? $hParams->{'Subject'} : '',
                 'FROM'              => $hParams->{'FROM'},
                 'REPLY'             => $hParams->{'REPLY'},
                 'TO'                => defined $hParams->{'TO'} ? SplitEMailAddresses($hParams->{'TO'}) : [],
                 'CC'                => defined $hParams->{'CC'} ? SplitEMailAddresses($hParams->{'CC'}) : [],
                 'BCC'               => defined $hParams->{'BCC'}? SplitEMailAddresses($hParams->{'BCC'}) : [],
                 'SkipBadRecipients' => $hParams->{'SkipBadRecipients'}
                };

    my $Mail = DE_EPAGES::Mail::API::Mail->new($hInfo);
    my $Site = $Object->getSite();
    my $BaseURL = $Site->class->alias eq 'Shop' && $Site->get('IsUnityRegistered') ? $Site->get('WebUrlUnitySSL') : $Site->get('WebUrlSSL');
    $hParams->{'Content'} = ReplaceURIWithURL($hParams->{'Content'}, $BaseURL);

    if ($hParams->{'HTMLAllowed'}) {
        # create the HTML Mail with the alternate text
        $Mail->addHTML({'content'        => $hParams->{'Content'},
                        'encoding'       => 'quoted-printable',
                        'type'           => 'text/html',
                        'type.charset'   => 'utf-8',
                        'html2plain'     => 1,
                        'alttext'        => '',
                        'altencoding'    => 'quoted-printable',
                        'alt.type.charset' => 'utf-8',
                       });

    } else {
        # create a text only mail
        $Mail->addText({'content'     => $Mail->convertHtml2Plain($hParams->{'Content'}),
                        'encoding'    => 'quoted-printable',
                        'type'        => 'text/plain',
                        'type.charset'=> 'utf-8',
                        });
    }
    if ($hParams->{'Attachments'}) {
        $Mail->addAttachment($hParams->{'Attachments'}) ;
    }

    TriggerHook('MailTypeGetMail',{
        'Object'   => $Object,
        'Mail'     => $Mail,
        'MailType' => $self,
    });
    return $Mail;
}

#========================================================================================
# function     _subject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Subject = $MailType->_subject( $Object, $Subject, $hRegionalParams, $hVars );
# example      $Subject = $MailType->_subject( $System, 'Hallo #FirstName', $hRegionalParams, $hVars );
#----------------------------------------------------------------------------------------
# description  The TLE placeholders in the mail template subject
#               are replaced by the variables given in $hVars.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $Subject | user-specified subject (overrides template) | string
# input        $hRegionalParams | regional options
#                       <ul>
#                         <li>Locale</li>
#                         <li>Language</li>
#                       </ul> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# return       $Subject | subject | string
#========================================================================================
sub _subject {
    my $self = shift;
    my ($Object, $Subject, $hRegionalParams, $hVars) = @_;

    return $Subject if defined $Subject;

    my $LanguageID = GetPKeyLanguageByCode($hRegionalParams->{'Language'});
    return $self->replaceTLEs($Object, $self->get('Subject', $LanguageID), $hRegionalParams, $hVars);
}

#========================================================================================
# function     replaceTLEs
# state        public
#----------------------------------------------------------------------------------------
# syntax       $String = $MailType->replaceTLEs( $Object, $String, $hRegionalParams, $hVars );
# example      $String = $MailType->replaceTLEs( $System, 'Hello #FirstName', $hRegionalParams, $hVars );
#----------------------------------------------------------------------------------------
# description  The TLE placeholders in the string
#               are replaced by the variables given in $hVars.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $String | template string with TLEs | string
# input        $hRegionalParams | regional options
#                       <ul>
#                         <li>Locale</li>
#                         <li>Language</li>
#                       </ul> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# return       $String | string with replaced TLE variables | string
#========================================================================================
sub replaceTLEs {
    my $self = shift;
    my ($Object, $String, $hRegionalParams, $hVars) = @_;
    TestHash( 'hRegionalParams', $hRegionalParams );
    TestHash( 'hVars', $hVars );

    $self->_tleContext( $Object, $hRegionalParams, $hVars, sub {
        my ($hVarsObject, $LanguageID) = @_;;
        $String = ReplaceTLEs($String, $hVarsObject);
    });

    return $String;
}

#========================================================================================
# function     content
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Content = $MailType->content( $Object, $Content, $hRegionalParams, $hVars );
# example      $Content = $MailType->content( $System, undef, $hRegionalParams, $hVars );
#----------------------------------------------------------------------------------------
# description  The TLE placeholders in the mail template content
#               are replaced by the variables given in $hVars.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $Content | user-specified content (overrides template) | string
# input        $hRegionalParams | regional options
#                       <ul>
#                         <li>Locale</li>
#                         <li>Language</li>
#                       </ul> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# input        $SkipAdditionalAttributes | include AdditionalText and Header attributes | boolean (optional)
# return       $Content | content | string
#========================================================================================
sub content {
    my $self = shift;
    my ($Object, $Content, $hRegionalParams, $hVars, $SkipAdditionalAttributes) = @_;

    TestHash( 'hRegionalParams', $hRegionalParams );
    TestHash( 'hVars', $hVars );

    return $Content if defined $Content;

    $self->_tleContext( $Object, $hRegionalParams, $hVars, sub {
        my ($hVarsObject) = @_;
        my $PageType = $self->getPageType($Object);
        my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );
        my $Template = $SkipAdditionalAttributes ? 'Content' : undef;
        $Content = ProcessPage($Object, $ObjectPageType, $hRegionalParams->{'Language'}, $hVarsObject, undef, undef, $Template);
    });
    return $Content;
}

#========================================================================================
# function     getPageType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageType = $MailType->getPageType();
#----------------------------------------------------------------------------------------
# description  Returns the page type related to the current mail type.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# return       $PageType   | PageType | PageType object
#========================================================================================

sub getPageType {
    my $self = shift;
    my ($Object) = @_;

    return $self->get('MailTypeTemplate')->get('PageType');
}

#========================================================================================
# function     _tleContext
# state        private
#----------------------------------------------------------------------------------------
# syntax       $MailType->_tleContext( $Object, $hRegionalParams, $hVars, $Sub );
#----------------------------------------------------------------------------------------
# description  Initializes the TLE hash and sets the language context for the TLE
#               processor to the language given in $hRegionalParams->{Language}.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $hRegionalParams | see <function _subject> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# input        $Sub | code to execute in the language context. The code is called
#                   with 2 parameters:
#                   <ul>
#                       <li>$hVarsObject - TLE hash, including MailType and regional settings - ref.hash</li>
#                       <li>$LanguageID - language id - int</li>
#                   </ul> | ref.code
#========================================================================================
sub _tleContext {
    my $self = shift;
    my ($Object, $hRegionalParams, $hVars, $Sub) = @_;

    my $hVarsObject = $Object->tleHash;
    $hVarsObject->{'MailType'} = $self->tleHash;
    $hVarsObject->{$_} = $hVars->{$_} foreach keys %$hVars;
    $hVarsObject->{'INPUT'} = {
        'ObjectID' => $Object->id,
        %$hRegionalParams
    };

    my $LanguageID = GetPKeyLanguageByCode( $hRegionalParams->{'Language'} );
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
    $Sub->( $hVarsObject, $LanguageID );
    return;
}

#========================================================================================
# function     _htmlAllowed
# state        private
#----------------------------------------------------------------------------------------
# syntax       $HTMLAllowed = $MailType->_htmlAllowed( $HTMLAllowed );
#----------------------------------------------------------------------------------------
# description  returns flag for mail as HTML (default: 0=plain text) - boolean
#----------------------------------------------------------------------------------------
# input        $HTMLAllowed  | HTMLAllowed (optional) | boolean
# return       $HTMLAllowed  | HTMLAllowed | boolean
#========================================================================================
sub _htmlAllowed {
    my $self = shift;
    my ($HTMLAllowed) = @_;
    return defined $HTMLAllowed ? $HTMLAllowed : 0;
}

#========================================================================================
# function     from
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FROM = $MailType->from;
# syntax       $FROM = $MailType->from( $FROM );
# example      $FROM = $MailType->from;
#----------------------------------------------------------------------------------------
# description  returns FROM address string of the mail
#----------------------------------------------------------------------------------------
# input        $FROM | mail 'from' field (optional) | string
# return       $FROM | mail 'from' field | string
#========================================================================================
sub from {
    my $self = shift;
    my ($FROM) = @_;
    return defined $FROM ? $FROM : $self->get('FROM');
}

#========================================================================================
# function     _reply
# state        private
#----------------------------------------------------------------------------------------
# syntax       $REPLY = $MailType->_reply( $REPLY );
#----------------------------------------------------------------------------------------
# description  returns REPLY address string of the mail
#----------------------------------------------------------------------------------------
# input        $REPLY | mail '_reply' field (optional) | string
# return       $REPLY | mail '_reply' field | string
#========================================================================================
sub _reply {
    my $self = shift;
    my ($REPLY) = @_;
    return defined $REPLY ? $REPLY : $self->get('FROM');
}

#========================================================================================
# function     to
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TO = $MailType->to;
# syntax       $TO = $MailType->to( $TO );
# example      $TO = $MailType->to;
#----------------------------------------------------------------------------------------
# description  returns TO address string of the mail
#----------------------------------------------------------------------------------------
# input        $TO | mail 'to' field (optional) | string
# return       $TO | mail 'to' field | string
#========================================================================================
sub to {
    my $self = shift;
    my ($TO) = @_;
    return defined $TO ? $TO : $self->get('TO');
}

#========================================================================================
# function     _cc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CC = $MailType->_cc($CC);
#----------------------------------------------------------------------------------------
# description  returns CC address string of the mail
#----------------------------------------------------------------------------------------
# input        $CC | mail 'cc' field (optional) | string
# return       $CC | mail 'cc' field | string
#========================================================================================
sub _cc {
    my $self = shift;
    my ($CC) = @_;
    return defined $CC ? $CC : $self->get('CC');
}

#========================================================================================
# function     _bcc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $BCC = $MailType->_bcc($BCC);
# example      $BCC = $MailType->_bcc($BCC);
#----------------------------------------------------------------------------------------
# description  returns BCC address string of the mail
#----------------------------------------------------------------------------------------
# input        $BCC | mail 'bcc' field (optional) | string
# return       $BCC | mail 'bcc' field | string
#========================================================================================
sub _bcc {
    my $self = shift;
    my ($BCC) = @_;
    return defined $BCC ? $BCC : $self->get('BCC');
}

#========================================================================================
# function     joinMailAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EMailAddress = $MailType->joinMailAddress($Name, $EMail);
# example      $EMailAddress = $MailType->joinMailAddress("Max Mustermann", "m.mustermann@epages.de");
#               $EMailAddress eq '"Max Mustermann" <m.mustermann@epages.de>'
#----------------------------------------------------------------------------------------
# description  joins the name and email to an email-address
#----------------------------------------------------------------------------------------
# input        $Name | name of email user | string
# input        $EMail | email | string
# return       $EMailAddress | joined name and email | string
#========================================================================================
sub joinMailAddress {
    my $self = shift;
    my ($Name, $EMail) = @_;
    return $EMail if $EMail =~ /</ || !$Name;
    $Name =~ s/\\/\\\\/g; # escape backslashes
    $Name =~ s/"/\\"/g; # escape double quotes
    return sprintf '"%s" <%s>', $Name, $EMail;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::MailType::API::Object::MailType->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    my $MailTypeTemplate = defined $hInfo->{'MailTypeTemplate'} ? $hInfo->{'MailTypeTemplate'} :undef;
    my $IsActive = defined $MailTypeTemplate ? $MailTypeTemplate->get('IsAlwaysActive') : 0;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsActive' => $IsActive
    };
}

1;
