use strict;
use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Order::API::Cartridge;
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Order::API::SalesVolume qw ( GetDeliveryThresholdSalesVolume );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObject ExistsObject );
use DE_EPAGES::Order::API::Shop qw ( TriggerMailEventDeliveryThresholdReached );
use DE_EPAGES::Core::API::PerlTools qw( AddDuration );
use Date::Format qw ( time2str );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

#========================================================================================
# function         CalculateSalesVolume
# state            public
#----------------------------------------------------------------------------------------
# syntax           CalculateSalesVolume();
#----------------------------------------------------------------------------------------
# description      Calculates sales volume for every Shop and trigger mail event
#                   if salesVolume is reached
#========================================================================================
sub CalculateSalesVolume {

    my $System = LoadRootObject();
    my $aShopIDs = $System->child('Shops')->findChildren;

    foreach my $ShopID ( @$aShopIDs ) {
        return unless ExistsObject($ShopID);
        my $Shop = LoadObject($ShopID);

        my $Now = GetCurrentDBHandle()->currentDateTime;
        my $dt = time2str($Now->epoch);

        # calculate sales volume
        my $StartOfCurrentYear = $Now->clone->truncate( to => 'year');
        my $EndOfCurrentYear = AddDuration($StartOfCurrentYear->clone, 'years', 1);
        my $SalesVolume = GetDeliveryThresholdSalesVolume($Shop, $StartOfCurrentYear, $EndOfCurrentYear);

        my $hDeliveryThresholdSalesVolume = $Shop->get('DeliveryThresholdSalesVolume');

        # Check if DeliveryThresholdSalesVolume is already set and take LimitReachedMailSent attr
        # otherwise its the first run so create initial
        if (defined $hDeliveryThresholdSalesVolume->{'LimitReachedMailSent'}) {
            $hDeliveryThresholdSalesVolume = {
                'SalesVolume'          => $SalesVolume,
                'LastUpdated'          => $dt,
                'LimitReachedMailSent' => $hDeliveryThresholdSalesVolume->{'LimitReachedMailSent'},
            };
        } else {
            $hDeliveryThresholdSalesVolume = {
                'SalesVolume'          => $SalesVolume,
                'LastUpdated'          => $dt,
                'LimitReachedMailSent' => 0,
            };
        }

        $Shop->set({'DeliveryThresholdSalesVolume' => $hDeliveryThresholdSalesVolume});

        my $hParams = {
            'Shop' => $Shop,
        };
        TriggerMailEventDeliveryThresholdReached($hParams);
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { CalculateSalesVolume(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1 unless $NoOutput;
}

sub usage {
    print <<END_USAGE;
Description:
    Calculates sales volume for every shop

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -nooutput   do not print any output

flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
