#========================================================================================
# package      DE_EPAGES::Product::API::Object::LineItemProduct
# state        public
# base         DE_EPAGES::Order::API::Object::LineItemSales
# description  object interface for LineItemProduct.
#========================================================================================
package DE_EPAGES::Product::API::Object::LineItemProduct;
use base DE_EPAGES::Order::API::Object::LineItemSales;

use strict;
use DateTime;
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf GenerateGUID);
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Product::API::Table::LineItemProductMaxDownload qw (
    DeleteLineItemProductMaxDownload GetPKeyLineItemProductMaxDownloadsByLineItemProductID
);
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestString );
## no critic qw(RequireCorrectImports)
use DE_EPAGES::Product::API::XML::LineItemProductDeleteHandler;
use DE_EPAGES::Product::API::XML::LineItemProductExportDriver;
use DE_EPAGES::Product::API::XML::LineItemProductImportHandler;
## use critic
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Object::API::Constants qw( POSITION_START POSITION_STEP );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemProduct->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'LineItemProduct' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $LineItemProduct->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemProduct = DE_EPAGES::Product::API::Object::LineItemProduct->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemProduct');
#               my $LineItemProduct = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Overwrites standard <function DE_EPAGES::Order::API::Object::LineItem::insert>
#               to call OBJ_InsertLineItemProduct hook.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemProduct | new object | object
# hook         OBJ_InsertLineItemProduct | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemProductID - line item product id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    #-- renumber children
    my $Container = $self->parent();
    if ( $Container && $Container->instanceOf('LineItemContainer') ) {
        my $aLineItemProducts = $Container->childrenByClass('LineItemProduct');

        $aLineItemProducts = [
            map  { $_ }
            sort { $a->id cmp $b->id }
            @$aLineItemProducts
        ];

        my $Position = POSITION_START;
        foreach my $Object ( @$aLineItemProducts ) {
            $Object->set( { Position => $Position } );
            $Position += POSITION_STEP;
        }
    }

    TriggerHook('OBJ_InsertLineItemProduct', {'Object' => $self, 'LineItemProductID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Product OrderUnit Weight WeightUnit WeightQuantity Prepayment EcoParticipation MaxDownloadTime EndDownloadTime ); ## no critic qw(ProhibitLongLines)

    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemProduct->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemProduct' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemProduct | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item id         - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;
    if ($self->get('IsLineItemProductBundle')) {
        TriggerHook('OBJ_DeleteLineItemProductBundle', {'Object' => $self, 'LineItemID' => $self->id });
    } else {
        TriggerHook('OBJ_DeleteLineItemProduct', {'Object' => $self, 'LineItemID' => $self->id });
    }
    my $LineItemDeposit = $self->get('LineItemDeposit');
    $LineItemDeposit->delete if defined $LineItemDeposit and $self->parent->parent->instanceOf('Basket');
    $self->set({
        'Product' => undef,
        'OrderUnit' => undef,
        'WeightUnit' => undef,
    });

    # delete lineitemproductmaxdownload
    DeleteLineItemProductMaxDownload($_) foreach @{GetPKeyLineItemProductMaxDownloadsByLineItemProductID($self->id)};

    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->set($hValues, $LanguageID);
# example      $Object->set({ 'LocaleID' => 'de_DE' });
# example      $Object->set({ 'Name' => 'Fuball' }, GetPKeyLanguageByCode('de'));
# example      $Object->set({ 'Name' => 'Soccer' }, GetPKeyLanguageByCode('en'));
#----------------------------------------------------------------------------------------
# description  Sets the given attribute values on the object.
#               This function triggers hooks 'OBJ_BeforeUpdateObject' and 'OBJ_AfterUpdateObject'
#               and any specialized OBJ_BeforeUpdate* and OBJ_AfterUpdate* hooks.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional) language id for localized attributes | integer
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateObject | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectID      - object id            - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateObject | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectID      - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set { ##no critic qw(ProhibitAmbiguousNames)
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    $self->SUPER::set($hValues, $LanguageID);

    my $BundledProductContainer = $self->get('BundledProductContainer');
    return unless defined $BundledProductContainer;

    # update bundlecontainer: quantity and products of container should be the same like bundleproductmaps
    my $Product = $self->get('Product');
    if ( defined $Product and $Product->get('IsBundleProduct') and $self->container->parent->instanceOf('Basket') ) {

        # update quantities and remove if bundledproductmap is removed
        my $BundledItems = $BundledProductContainer->get('LineItems');
        my $BundleProductMaps = $Product->get('BundleProductMaps');
        foreach my $BundledItem( @$BundledItems ) {
            my $BundledItemProduct = $BundledItem->get('Product');
            my $found = 0 ;
            if (defined $BundledItemProduct) {
                foreach my $BundleProductMap (@$BundleProductMaps ) {
                    if ($BundleProductMap->get('BundledProduct')->id == $BundledItemProduct->id ) {
                        $found = 1;
                        $BundledItem->set({
                            'Quantity' => $BundleProductMap->get('BundledQuantity'),
                            'Position' => $BundleProductMap->get('Position'),
                        });
                        last;
                    }
                }
            }
            $BundledItem->delete unless $found;
        }

        # insert bundledproductmap which is added at BO
        $BundledItems = $BundledProductContainer->get('LineItems');
        if ( scalar (@$BundledItems) != scalar (@$BundleProductMaps )) {
            foreach my $BundleProductMap (@$BundleProductMaps ) {
                my $BundledProduct = $BundleProductMap->get('BundledProduct');
                my  $found = 0 ;
                foreach my $BundledItem( @$BundledItems ) {
                    my $BundledItemProduct = $BundledItem->get('Product');
                    if ($BundledProduct->id == $BundledItemProduct->id ) {
                        $found = 1;
                        last;
                    }
                }
                #add if not included yet
                $BundledProduct->addToContainer($BundledProductContainer, {
                    'Quantity' => $BundleProductMap->get('BundledQuantity'),
                    'Position' => $BundleProductMap->get('Position'),
                }) unless $found;
            }
        }
    }
    return;
}


#========================================================================================
# function     clone
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Clone = $LineItem->clone( $hInfo );
# syntax       $Clone = $LineItem->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new LineItem into the database as copy from old LineItem.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>ClassID - class id - int</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>ParentID - parent objectid - int</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# input        $CloneFiles | clone files associated with objects (optional, default is true) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive, $CloneFiles) = @_;
    $Recursive //= 0;

    my $Clone = $self->SUPER::clone($hInfo, $Recursive, $CloneFiles);
    if ($self->get('IsLineItemProductBundle')) {
        # correct the BundledProductContainer
        $Clone->set({'BundledProductContainer' => $Clone->child('BundledProductContainer')});
    }
    else {
        # correct the VariationString
        $Clone->set({'VariationString' => $self->get('VariationString')}) ;
    }

    if ($self->get('IsLineItemDownloadProduct')) {
        my $hMaxDownloadCounts = $self->get('MaxDownloadCounts');
        if ($hMaxDownloadCounts) {
            $Clone->set({'MaxDownloadCounts' => $hMaxDownloadCounts});
        }

        my $hDownloadLinkGUIDs = {};
        my $aDownloadProductMaps = $self->get('Product')->get('DownloadProductMaps');
        if ($aDownloadProductMaps) {
            foreach my $DownloadProductMap (@$aDownloadProductMaps) {
                $hDownloadLinkGUIDs->{$DownloadProductMap->get('DownloadProductMapID')} = GenerateGUID();
            }
            $Clone->set({'DownloadLinkGUIDs' => $hDownloadLinkGUIDs});
        }
    }

    return $Clone;
}

#========================================================================================
# function     getExtraExportableAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributes = $Object->getExtraExportableAttributes();
#----------------------------------------------------------------------------------------
# description  Returns names of attributes of object for XML export of orders.
#----------------------------------------------------------------------------------------
# return       $aAttributes | plain attributes | ref.array.string
#========================================================================================
sub getExtraExportableAttributes {
    return [qw( VariationString EndDownloadTime )];
}

#========================================================================================
# function     getExtraContainerAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributes = $Object->getExtraContainerAttributes();
#----------------------------------------------------------------------------------------
# description  Returns names of lineitemcontainer attributes of object for XML export of
#               orders. (e.g. BundleProductContainer)
#----------------------------------------------------------------------------------------
# return       $aAttributes | plain attributes | ref.array.string
#========================================================================================
sub getExtraContainerAttributes {
    my $self = shift;
    return $self->get('IsLineItemProductBundle') ? [qw(BundledProductContainer)] : [];
}

#========================================================================================
# function     recalculate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemProduct->recalculate($Container, $hValues);
#----------------------------------------------------------------------------------------
# description  Calls
#               <function DE_EPAGES::Product::API::Object::Product::recalculateLineItem>
#               to calculate the line item price based on product price and quantity.
#               Updates the LineItemPrice attribute of the line item.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
# input        $hValues | values that have changed | ref.hash
#========================================================================================
sub recalculate {
    my $self = shift;
    my ($Container, $hValues) = @_;
    my $Product = $self->get('Product');
    if (defined $Product and $Container->parent->instanceOf('Basket')) {
        eval { $Product->recalculateLineItem($self, $hValues); };
        if (ExistsError()) {
            my $Error = GetError();
            if (ElementOf($Error->code, 'ProductNoPrice', 'ProductCanNotChangedAtBasket')) {
                $self->delete;
                Warning($Error);
            }
            else {
                Error( $Error );
            }
        }
    } else { # order or external product
        my $Discount = $self->get('Discount');
        $Discount = 0 unless defined $Discount;

        my $LocaleID = $Container->get('LocaleID');
        my $Locale = $Container->getSite()->siteLocale($LocaleID);

        my $NoTax = $Container->get('NoTax');
        my $TaxRate;
        # we have product with digital tax class
        if ($self->get('TaxClassDigital')) {
            my $TaxClass  = $self->get('TaxClassDigital');
            my $TaxMatrix = $Container->get('TaxMatrixDigital');
            my $TaxArea = $Container->get('TaxAreaDigital');
            $TaxRate = defined $TaxArea
                ? ($NoTax ? 0 : $TaxMatrix->taxRateDigital($TaxArea, $TaxClass))
                : $self->get('TaxRate');

            # test if we have an exclusion by address and set prices to 0
            $TaxRate = 0 unless $Container->get('BasketOrOrder')->taxAreaDigitalByAddress();

        } else{
            my $TaxClass  = $self->get('TaxClass');
            my $TaxArea = $Container->get('TaxArea');
            $TaxRate = defined $TaxArea
                ? ($NoTax ? 0 : $Container->taxRate($TaxClass))
                : $self->get('TaxRate');
        }
        $self->set({
            'LineItemPrice' => $Locale->roundMoney($self->get('BasePrice') * $self->get('Quantity') - $Discount),
            'TaxRate'       => $TaxRate,
        });
    }
    return;
}

#========================================================================================
# function     addQuantityToStock
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemProduct->addQuantityToStock;
#----------------------------------------------------------------------------------------
# description  Calls <function DE_EPAGES::Product::API::Object::Product::addToStock>
#               with quantity of lineitem. If the line item contains no product, the
#               stock cant be changed.
#========================================================================================
sub addQuantityToStock {
    my $self = shift;
    if ($self->get('IsLineItemProductBundle')) {
        foreach my $LineItem ( @{$self->get('BundledProductContainer')->get('LineItems')} ) {
            my $Product = $LineItem->get('Product');
            if ( defined $Product ) {
                $Product->addToStock($self->get('Quantity') * $LineItem->get('Quantity'));
            }
        }
    } else {
        my $Product = $self->get('Product');
        if ( defined $Product ) {
            $Product->addToStock($self->get('Quantity'));
        }
    }
    return;
}

#========================================================================================
# function     substractQuantityFromStock
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemProduct->substractQuantityFromStock;
#----------------------------------------------------------------------------------------
# description  Calls <function DE_EPAGES::Product::API::Object::Product::substractFromStock>
#               with quantity of lineitem. If the line item contains no product, the
#               stock cant be changed.
#========================================================================================
sub substractQuantityFromStock {
    my $self = shift;
    if ($self->get('IsLineItemProductBundle')) {
        foreach my $LineItem ( @{$self->get('BundledProductContainer')->get('LineItems')} ) {
            my $Product = $LineItem->get('Product');
            if ( defined $Product ) {
                $Product->substractFromStock($self->get('Quantity') * $LineItem->get('Quantity') );
            }
        }
    } else {
        my $Product = $self->get('Product');
        if ( defined $Product ) {
            $Product->substractFromStock($self->get('Quantity'));
        }
    }
    return;
}

#========================================================================================
# function     isQuantityCorrectOnInsertOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemProduct->isQuantityCorrectOnInsertOrder;
#----------------------------------------------------------------------------------------
# description  Calls <function DE_EPAGES::Product::API::Object::Product::lineItem>
#               to get the possible quantity of lineitem.
#               If the line item contains no product the quantity is considered to be
#               available.
#               Returns true if the desired quantity is available and is valid according
#               to the products minimum order amount and order amount interval.
#----------------------------------------------------------------------------------------
# return       $IsValid | true if the quantity is ok | boolean
#========================================================================================
sub isQuantityCorrectOnInsertOrder {
    my $self = shift;

    my $Container = $self->container;
    my $Product = $self->get('Product');
    return 0 unless defined $Product;

    my $hLineItemInfo = $Product->lineItem($Container, {'LineItem' => $self,
                                                        'Container'=> $Container,
                                                        'Quantity' => $self->get('Quantity'),});
    # reduced if not enough products are available
    return (defined $hLineItemInfo->{'IsChanged'} and $hLineItemInfo->{'IsChanged'} == 1 )? 0 : 1;
}

#========================================================================================
# function     grepAvailableShippingMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShippingMethods = $LineItem->grepAvailableShippingMethods($aShippingMethods);
#----------------------------------------------------------------------------------------
# description  Reduce available shipping methods, if shipping methods selected for product.
#----------------------------------------------------------------------------------------
# input        $aShippingMethods | shipping methods | ref.array.object
# return       $aShippingMethods | selected shipping methods | ref.array.object
#========================================================================================
sub grepAvailableShippingMethods {
    my $self = shift;
    my ($aShippingMethods) = @_;

    return $aShippingMethods unless $self->getSite->featureMaxValue('ShippingMethodsPerProduct');

    my $Product = $self->get('Product');
    return $aShippingMethods unless defined $Product;

    my $ahItemShippingMethods = $Product->getWithParent('ShippingMethods');
    return $aShippingMethods unless scalar @$ahItemShippingMethods;

    my @IDs = map { $_->id } @$ahItemShippingMethods;
    return [ grep { ElementOf($_->id, @IDs) } @$aShippingMethods];
}

#insertLineItemCustomizedText
sub insertLineItemCustomizedText {
    my $self = shift;
    my ($hParams)= @_;
    TestHash('hParams', $hParams, {
        'Text' => 'String;NotNull',
        'FontName' => 'String;NotNull',
        'FontSize' => 'Float;NotNull',
        'Color' => 'String;NotNull',
        'Width' => 'Float;NotNull',
        'Height' => 'Float;NotNull',
        'InnerWidth' => 'Float;NotNull',
        'InnerHeight' => 'Float;NotNull',
        'FromTop' => 'Integer;NotNull',
        'FromLeft' => 'Integer;NotNull',
        'ZIndex' => 'Integer;NotNull',
        'Orientation' => 'Integer;NotNull',
        'AreaID' => 'Integer;NotNull',
    });

    my $Folder = $self->folder('LineItemCustomizedTexts');

    $hParams->{'Alias'} = $Folder->nextAlias('Area'.$hParams->{'AreaID'}.'_LineItemCustomizedText%s');
    $hParams->{'LineItemID'} = $self->id;
    my $LineItemCustomizedText = $Folder->insertChild('LineItemCustomizedText', $hParams);
    $LineItemCustomizedText->set({'Text' => $hParams->{'Text'}});

    return $LineItemCustomizedText;
}

#deleteLineItemCustomizedText
sub deletetLineItemCustomizedText {
    my $self = shift;
    my ($Alias)= @_;
    TestString('Alias', $Alias);

    if ($self->existsLineItemCustomizedText($Alias)) {
        my $Folder = $self->folder('LineItemCustomizedTexts');
        $Folder->child($Alias)->delete;
    }
    return;
}

#existsLineItemCustomizedText
sub existsLineItemCustomizedText {
    my $self = shift;
    my ($Alias)= @_;
    TestString('Alias', $Alias);
    return $self->folder('LineItemCustomizedTexts')->existsChild($Alias);
}

#insertLineItemCustomizedFile
sub insertLineItemCustomizedFile {
    my $self = shift;
    my ($hParams)= @_;
    TestHash('hParams', $hParams, {
        'FileName' => 'String;NotNull',
        'Width' => 'Float;NotNull',
        'Height' => 'Float;NotNull',
        'InnerWidth' => 'Float;NotNull',
        'InnerHeight' => 'Float;NotNull',
        'FromTop' => 'Integer;NotNull',
        'FromLeft' => 'Integer;NotNull',
        'ZIndex' => 'Integer;NotNull',
        'Orientation' => 'Integer;NotNull',
        'AreaID' => 'Integer;NotNull',
    });


    my $Folder = $self->folder('LineItemCustomizedFiles');

    $hParams->{'Alias'} = $Folder->nextAlias('Area'.$hParams->{'AreaID'}.'_LineItemCustomizedFile%s');
    $hParams->{'LineItemID'} = $self->id;
    my $LineItemCustomizedFile = $Folder->insertChild('LineItemCustomizedFile', $hParams);

    return $LineItemCustomizedFile;
}

#deleteLineItemCustomizedFile
sub deletetLineItemCustomizedFile {
    my $self = shift;
    my ($Alias)= @_;
    TestString('Alias', $Alias);

    if ($self->existsLineItemCustomizedFile($Alias)) {
        my $Folder = $self->folder('LineItemCustomizedFiles');
        $Folder->child($Alias)->delete;
    }
    return;
}

#existsLineItemCustomizedFile
sub existsLineItemCustomizedFile {
    my $self = shift;
    my ($Alias)= @_;
    TestString('Alias', $Alias);
    return $self->folder('LineItemCustomizedFiles')->existsChild($Alias);
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Product::API::Object::LineItemProduct->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsShippingRequired' => 1,
    };
}

1;
