#===========================================
# UNITY-7496
#   ProductAttributes are now from Class Attribute, but should be ProductAttribute
#===============================================
use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject LoadClassByAlias LoadObject );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

sub Main {

    local $| = 1;
    my ( $StoreName, $ShopAlias, $NoOutput, $TestMode, $Help );

    GetOptions(
        'help'        => \$Help,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
        'nooutput'    => \$NoOutput,
        'testmode'    => \$TestMode,
    );

    usage() if $Help or not $StoreName;

    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        Sub   => sub {
            my $aShopIDs = [];
            if ( $ShopAlias ) {
                $aShopIDs = [ LoadObjectByPath("/Shops/$ShopAlias")->id ];
            } else {
                $aShopIDs = LoadRootObject()->child('Shops')->findChildren();
            }

            my $ProductAttributeClass = LoadClassByAlias('ProductAttribute');
            my $AttributeClass = LoadClassByAlias('Attribute');
            print "TEST MODE: Nothing will be changed\n\n" if $TestMode;

            foreach my $ShopID ( @$aShopIDs ) {
                my $Shop = LoadObject($ShopID);
                #print "\n" . $Shop->pathString . "\n" if ! $NoOutput;

                #-- All product types, except
                #-- the base product class which does not seem to be affected, don't touch it
                my $aProductTypes = $Shop->folder('ProductTypes')->children();
                foreach my $ProductType ( @$aProductTypes ) {
                    #print '  ' . $ProductType->pathString . "\n" if ! $NoOutput;

                    my $aProductTypeAttributes = $ProductType->existsChild('Attributes') ? $ProductType->child('Attributes')->children() : [];
                    #-- ALL product type attributes
                    foreach my $PTAttribute ( @$aProductTypeAttributes ) {
                        next if $PTAttribute->class->id == $ProductAttributeClass->id; #-- This is the right attribute class

                        if ( $PTAttribute->class->id == $AttributeClass->id ) { #-- Should NOT be this attribute class
                            print $PTAttribute->pathString . "\n" if ! $NoOutput;
                            $PTAttribute->set({
                                'Class' => $ProductAttributeClass,
                            }) if ! $TestMode;
                        } else {
                            print $PTAttribute->pathString . " (UNKNOWN CLASS " . $PTAttribute->class->alias . ")\n" if ! $NoOutput;
                        }
                    }
                }
            }
            SynchronizeCache();
        },
    );

    my $time2 = time;
    printf "\nneed %.3f seconds\n", $time2 - $time1;

}

sub usage {
    print <<END_USAGE;
Description:
    Fix the class of product type attribute objects, from 'Attribute' to 'ProductAttribute'

Usage:
    perl $0 -storename <STORE> [-shop DemoShop] [-testmode] [-nooutput]

options:
    -storename  name of store                                       required
    -shop       alias of shop                                       optional
    -testmode   just print output (if on), don't change anything    optional
    -nooutput   no output                                           optional

Example:
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main, );

1;
