#========================================================================================
# package      DE_EPAGES::Shop::API::Object::ShopLanguage
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for ShopLanguage.
#========================================================================================
package DE_EPAGES::Shop::API::Object::ShopLanguage;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Object::API::Factory  qw ( LoadClassByAlias );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShopLanguage->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShopLanguage' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShopLanguage->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopLanguage = DE_EPAGES::Shop::API::Object::ShopLanguage->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShopLanguage');
#               my $ShopLanguage = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShopLanguage' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShopLanguage | new object | object
# hook         OBJ_InsertShopLanguage | hook param eter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopLanguageID - object id            - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    # Increment the usage counter of the language feature.
    my $Shop = $hInfo->{'Shop'} || $hInfo->{Parent}->getSite();
    my $CallFromUI = $hInfo->{'FromUI'};

    delete $hInfo->{'FromUI'};

    my $DoSwitch = 0;
    unless ($CallFromUI) {
        if ($Shop->featureMaxValue('Languages') == 1) {
            # Switch languages.
            my $ShopLanguageID = $Shop->get('LanguageID');
            if (defined $ShopLanguageID) {
                if ($Shop->existsLanguage($ShopLanguageID)) {
                    $Shop->removeLanguage($ShopLanguageID);
                    $DoSwitch = 1;
                }
            }
        }
    }

    # isUnity check is only taken into account if shop is already created
    # onShop creation this flag is not set yet
    $Shop->isUnity() ? '' : $Shop->featureIncrementValue('Languages');

    my $self = $class->SUPER::insert($hInfo);

    if ($DoSwitch) {
        # Activate language and set it default.
        my $NewLanguageID = $hInfo->{'LanguageID'};
        my $LocaleID = GetInfoLanguage($NewLanguageID)->{'DefaultLocaleID'};
        $Shop->addLocale($LocaleID);
        $Shop->set({ 'LocaleID' => $LocaleID });
    }
    TriggerHook('OBJ_InsertShopLanguage', {'Object' => $self, 'ShopLanguageID' => $self->id });

    delete $hInfo->{$_} foreach qw ( Shop LanguageID ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopLanguage->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShopLanguage' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShopLanguage | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopLanguageID - object id            - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShopLanguage', {'Object' => $self, 'ShopLanguageID' => $self->id });

    # Delete SiteLocales of this language.
    my $aLocales = $self->get('Locales');
    my $Shop = $self->getSite();
    $Shop->removeLocale($_->{'LocaleID'}) foreach @$aLocales;

    # Delete shop language.
    $self->SUPER::delete;

    # Decrement the usage counter of the Language feature.
    if (!$Shop->isUnity()) {
        $Shop->featureIncrementValue('Languages', -1);
    }
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopLanguage->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShopLanguage' and 'OBJ_AfterUpdateShopLanguage'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShopLanguage | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopLanguageID - object id            - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShopLanguage | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopLanguageID - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShopLanguage', {'Object'=>$self, 'Values'=> $hValues, 'ShopLanguageID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShopLanguage',  {'Object'=>$self, 'Values'=> $hValues, 'ShopLanguageID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     importXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    my $XMLParent = $Handler->topID('Object');
    my $Shop = $XMLParent->getSite();
    my $LanguageID = GetPKeyLanguageByCode($hAttributes->{'Language'});
    my ($ShopLanguage) = @{LoadClassByAlias('ShopLanguage')->getObjects('ByShopIDLanguageID', [$Shop->id, $LanguageID])};
    $ShopLanguage = $Shop->addLanguage($LanguageID) unless defined $ShopLanguage;
    my $Position = $hAttributes->{'Position'};
    $ShopLanguage->set({'Position' => $Position}) if defined $Position;
    return $ShopLanguage;
}

sub exportXMLAttributes {
    my $self = shift;
    return {'Language'=>$self->alias};
}

1;
