#========================================================================================
# package      DE_EPAGES::ShopMailType::API::ShopMailType
# state        public
# base         Exporter
# description  object interface for ShopMailType.
#========================================================================================
package DE_EPAGES::ShopMailType::API::ShopMailType;
use base Exporter;
use strict;

our @EXPORT_OK = qw(
    GetShopMailType
    GetUnityShopToAdminMailTypes
    GetUnityShopToUserMailTypes
);

use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );


#========================================================================================
# function     GetShopMailType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopMailType = GetShopMailType($Shop, $Alias)
# syntax       $ShopMailType = GetShopMailType($Shop, $Alias, $IfActive)
# example      my $ShopMailType = GetShopMailType($Shop, 'CustomerWelcome', 1);
#               if( defined $ShopMailType ) {
#                   $ShopMailType->send( $Shop, {
#                           'TO' => $User->get('EMail'),
#                       }, {
#                           Locale   => 'de_DE',
#                           Language => 'de',
#                       }, {
#                           'Customer' => $Customer->tleHash,
#                           'User'     => $User->tleHash,
#                       }
#                   });
#               }
#----------------------------------------------------------------------------------------
# description  Returns a ShopMailType object of a shop.
#               If the object does not exist yet, it will be created from the
#               corresponding MailTypeTemplate object.
#               If the parameter $IfActive is true, then the function will return undef
#               if the mail event is not active.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $Alias | alias of the ShopMailType and MailTypeTemplate object | string
# input        $IfActive | tells the function to return undef if the ShopMailType does
#                   not exist or if it is not active | boolean
# return       $ShopMailType
#========================================================================================
sub GetShopMailType {
    my ($Shop, $Alias, $IfActive) = @_;
    $IfActive = 0 unless defined $IfActive;

    my $ShopMailType;
    my $Folder = $Shop->folder('MailTypes');
    if( $Folder->existsChild( $Alias ) ) {
        $ShopMailType = $Folder->child($Alias);
        return undef if $IfActive && !$ShopMailType->get('IsActive');
        my $From = $ShopMailType->from;
        if ($IfActive && $ShopMailType->get('IsActive') && (!$From || $From !~ /\S/)) {
            return undef;
        }

    }
    else {
        my $MailTypeTemplate = LoadRootObject()->folder('ShopMailTypeTemplates')->child($Alias);

        # create a ShopMailType object only if necessary
        return undef if $IfActive && !$MailTypeTemplate->get('IsAlwaysActive');

        $ShopMailType = $Folder->insertChild( 'ShopMailType', {
            'Alias'            => $Alias,
            'MailTypeTemplate' => $MailTypeTemplate,
            'Shop'             => $Shop,
            'Position'         => $MailTypeTemplate->get('Position'),
        });

        TriggerHook('API_ShopMailTypeCreated', {'ShopMailType'   => $ShopMailType });

    }

    # set the to address if not defined and the MailTypeTemplate has an TO field
    $ShopMailType->to;


    return $ShopMailType;

}

#========================================================================================
# function     GetUnityShopToAdminMailTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetUnityShopToAdminMailTypes($Shop)
# example      GetUnityShopToAdminMailTypes($Shop)
#----------------------------------------------------------------------------------------
# description  Returns all shop mailtypetemplate objects that is sent to the admin.
#----------------------------------------------------------------------------------------
# return       $array | mailtypetemplate objects | ref.array
#========================================================================================
sub GetUnityShopToAdminMailTypes {
    return [
            grep { $_->get('HasToField') == 1 && $_->get('IsForUnity') == 1 }
                @{LoadRootObject()->child('ShopMailTypeTemplates')->children}
    ];
}

#========================================================================================
# function     GetUnityShopToUserMailTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetUnityShopToUserMailTypes($MailTypeTemplateGroupItem)
# example      GetUnityShopToUserMailTypes($MailTypeTemplateGroupItem)
#----------------------------------------------------------------------------------------
# description  Returns all shop mailtypetemplate objects that is sent to the user.
#----------------------------------------------------------------------------------------
# input        $MailTypeTemplateGroup | MailTypeTemplateGroup object | object
# return       $array | MailTypeTemplate objects | ref.array
#========================================================================================
sub GetUnityShopToUserMailTypes{
    my ($MailTypeTemplateGroup) = @_;

    return [
            grep { $_->get('HasToField') == 0 && $_->get('IsForUnity') == 1 && $_->get('Alias') ne 'FulfillEbayTransaction'}
                @{$MailTypeTemplateGroup->getMailTypeTemplates}
    ];
}

1;
