#========================================================================================
# package      DE_EPAGES::ShopVote::API::TLE::ShopVoteHandler
# state        public
#----------------------------------------------------------------------------------------
# description Implements a TLE Function which returns preformated ShopVote widget HTML code
#========================================================================================
package DE_EPAGES::ShopVote::API::TLE::ShopVoteHandler;

use strict;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::ShopVote::API::Constants qw ( SEAL_VALIGN SEAL_HALIGN SEAL_VOFFSET SEAL_HOFFSET);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'ShopVoteHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::ShopVote::API::TLE::ShopVoteHandler;
# example      my $PrefixHandler  = new DE_EPAGES::ShopVote::API::TLE::ShopVoteHandle;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift;}

#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
# example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    my $Object = $Processor->tle('Object');
    my $Shop = $Object->getSite();

    if( $TLEName eq 'ExtractedShopVoteScriptCode' ) {
        my $ShopVoteScriptCode = $Shop->get('ShopVoteScriptCode');
        $ShopVoteScriptCode =~ /script src="([^"]*).*<script type="text\/javascript">(.*var myToken = "([a-zA-Z0-9]{0,32})".*)<\/script>/s;
        my $Source = $1;
        my $Code = $2;
        my $Token = $3;

        return {
            'Source' => $Source,
            'Code' => $Code,
            'Token' => $Token
        };
    }

    # extracts the display position values
    if( $TLEName eq 'ExtractedShopVoteSealPosition' ) {
        my $ShopVoteSealPosition = $Shop->get('ShopVoteSealPosition');

        return {
            'VAlign' => $ShopVoteSealPosition->{'VAlign'} // SEAL_VALIGN,
            'HAlign' => $ShopVoteSealPosition->{'HAlign'} // SEAL_HALIGN,
            'VOffset' => $ShopVoteSealPosition->{'VOffset'} // SEAL_VOFFSET,
            'HOffset' => $ShopVoteSealPosition->{'HOffset'} // SEAL_HOFFSET,
            'VOffsetDefault' => SEAL_VOFFSET,
            'HOffsetDefault' => SEAL_HOFFSET
        };
    }

    return undef;
}

#========================================================================================
# function     existsTLE
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->existsTLE( $Processor, $TLEName);
# example      my $Value = $Handler->existsTLE( $Processor, 'Products_Count');
#----------------------------------------------------------------------------------------
# description  This function is called by the tle processor to test if a TLE variable
#               is available.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | yes or no | bool
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    GetLog->debug( "ShopVoteHandler.existsTLE($TLEName)" );
    return (defined $self->tle($Processor, $TLEName)) ? 1 : 0;
}

sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'ShopVoteHandler.register' );
    $Processor->registerHandler('VariableHandler', $self, 'ExtractedShopVoteScriptCode');
    $Processor->registerHandler('VariableHandler', $self, 'ExtractedShopVoteSealPosition');
    return;
}
1;

